@@ -32,12 +32,9 @@
 DO_COMMAND(do_configure)
 {
 	char left[BUFFER_SIZE], right[BUFFER_SIZE];
-	struct listroot *root;
 	struct listnode *node;
 	int index;
 
-	root = ses->list[LIST_CONFIG];
-
 	arg = get_arg_in_braces(ses, arg, left,  FALSE);
 	arg = sub_arg_in_braces(ses, arg, right, GET_ONE, SUB_VAR|SUB_FUN);
 
@@ -152,12 +149,6 @@ DO_CONFIG(config_commandcolor)
 {
 	char buf[BUFFER_SIZE];
 
-	if (!is_color_code(arg))
-	{
-		tintin_printf2(ses, "#CONFIG COMMAND COLOR: Invalid color code '%s'.", arg);
-
-		return NULL;
-	}
 	substitute(ses, arg, buf, SUB_COL);
 
 	RESTRING(ses->cmd_color, buf);
