@@ -94,19 +94,19 @@ struct help_type help_table[] =
 	},
 	{
 		"ALIAS",
-		"<178>Command<078>: #alias <178>{<078>word<178>} {<078>commands<178>}<078>\n"
+		"<178>Command<078>: #alias <178>{<078>word<178>} {<078>command<178>}<078>\n"
 		"\n"
-		"         Defines a word that actually means a different word. Useful for long\n"
-		"         commands repeated often.  You can have multiple commands aliased to\n"
-		"         a single word, if you enclose the alias in braces.\n"
+		"         Defines a word that when entered executes the given command. If\n"
+		"         any arguments are given they are appended to the end of the command."
 		"\n"
-		"         If the commands include variables %0-99, these are substituted as\n"
+		"         If the command includes variables %0-99, these are substituted as\n"
 		"         part of the parsing.  %0 is set to all text after the word was input,\n"
 		"         %1 is set to the first word following the aliased word, and %2 is the\n"
-		"         second, and so on.\n"
+		"         second, and so on. If you use %0-99 no arguments are automatically\n"
+		"         appended.\n"
 		"\n"
 		"         The word can include the %1 to %99 variables making an alias behave\n"
-		"         like an action for complex input matching.\n"
+		"         like an action, allowing more complex input matching.\n"
 		"\n"
 		"<178>Example<078>: #alias gb get bread bag\n"
 		"         Typing gb at the prompt would be sent as 'get bread bag'.\n"
@@ -208,6 +208,7 @@ struct help_type help_table[] =
 		"         #chat {send}       {buddy|all} {text} Sends a raw data string\n"
 		"         #chat {sendfile}   {buddy} {filename} Start a file transfer\n"
 		"         #chat {serve}      {buddy}            Forward all public chat messages\n"
+		"         #chat {unitialize}                    Unitialize the chat port.\n"
 		"         #chat {who}                           Show all connections\n"
 		"         #chat {zap}        {buddy}            Close a connection\n"
 	},
@@ -259,11 +260,11 @@ struct help_type help_table[] =
 		"\n"
 		"         The tertiary colors are as follows:\n"
 		"\n"
-		"         <<888>acf> - Azure            <<888>afc> - Viridian\n"
-		"         <<888>caf> - Indigo           <<888>cfa> - Lime\n"
+		"         <<888>acf> - Azure            <<888>afc> - Jade\n"
+		"         <<888>caf> - Violet           <<888>cfa> - Lime\n"
 		"         <<888>fac> - Pink             <<888>fca> - Orange\n"
 		"\n"
-		"<178>Example<078>: #showme <<888>acf>Azure    <<888>afc>Viridian <<888>caf>Indigo\n"
+		"<178>Example<078>: #showme <<888>acf>Azure    <<888>afc>Jade     <<888>caf>Violet\n"
 		"<178>Example<078>: #showme <<888>cfa>Lime     <<888>fac>Pink     <<888>fca>Orange\n"
 	},
 	{
@@ -390,10 +391,10 @@ struct help_type help_table[] =
 	},
 	{
 		"END",
-		"<178>Command<078>: #end\n"
+		"<178>Command<078>: #end {<message>}\n"
 		"\n"
 		"         Terminates tintin and return to unix.  On most systems, ctrl-c has\n"
-		"         the same result.\n"
+		"         the same result. The message is optional.\n"
 	},
 	{
 		"ESCAPE CODES",
@@ -421,40 +422,42 @@ struct help_type help_table[] =
 		"         a brief description. Use #event %* to see the current list of available\n"
 		"         events you can use.\n"
 		"\n"
-		"         DATE                 %1 month - %3 day   %4 hour : %5 minute\n"
-		"         DAY                  %3 day\n"
+		"         DATE                  %1 month - %3 day   %4 hour : %5 minute\n"
+		"         DAY                   %3 day\n"
 		"         END OF PATH\n"
-		"         HOUR                 %4 hour\n"
+		"         HOUR                  %4 hour\n"
 		"         IAC <VAR> <VAR>\n"
-		"         IAC SB GMCP <MODULE> %0 data     %1 raw data\n"
-		"         IAC SB MSSP          %0 variable %1 value\n"
-		"         IAC SB MSDP          %0 variable %1 value\n"
-		"         IAC SB MSDP <VAR>    %1 value\n"
-		"         IAC SB NEW-ENVIRON   %0 variable %1 value\n"
-		"         IAC SB ZMP <VAR>     %0 value\n"
-		"         IAC SB <VAR>         %0 raw text %1 raw data\n"
-		"         MAP ENTER ROOM       %0 new vnum %1 old vnum\n"
-		"         MAP ENTER ROOM <VAR> %0 new vnum %1 old vnum\n"
-		"         MAP EXIT ROOM        %0 old vnum %1 new vnum\n"
-		"         MAP EXIT ROOM <VAR>  %0 old vnum %1 new vnum\n"
-		"         MINUTE               %5 minute\n"
-		"         MONTH                %1 month\n"
-		"         PROGRAM START        %0 client name %1 client version\n"
+		"         IAC SB GMCP <MODULE>  %0 data     %1 raw data\n"
+		"         IAC SB MSSP           %0 variable %1 value\n"
+		"         IAC SB MSDP           %0 variable %1 value\n"
+		"         IAC SB MSDP <VAR>     %1 value\n"
+		"         IAC SB NEW-ENVIRON    %0 variable %1 value\n"
+		"         IAC SB ZMP <VAR>      %0 value\n"
+		"         IAC SB <VAR>          %0 raw text %1 raw data\n"
+		"         MAP ENTER ROOM        %0 new vnum %1 old vnum\n"
+		"         MAP ENTER ROOM <VAR>  %0 new vnum %1 old vnum\n"
+		"         MAP EXIT ROOM         %0 old vnum %1 new vnum\n"
+		"         MAP EXIT ROOM <VAR>   %0 old vnum %1 new vnum\n"
+		"         MINUTE                %5 minute\n"
+		"         MONTH                 %1 month\n"
+		"         PROGRAM START         %0 client name %1 client version\n"
 		"         PROGRAM TERMINATION\n"
-		"         RECEIVED INPUT       %0 raw text\n"
-		"         RECEIVED LINE        %0 raw text %1 plain text\n"
-		"         RECEIVED OUTPUT      %0 raw text\n"
-		"         RECEIVED PROMPT      %0 raw text %1 plain text\n"
-		"         SCREEN RESIZE        %0 cols %1 rows\n"
-		"         SECOND               %6 second\n"
-		"         SEND OUTPUT          %0 raw text\n"
-		"         SESSION ACTIVATED    %0 name\n"
-		"         SESSION CONNECTED    %0 name %1 host %2 ip %3 port\n"
-		"         SESSION DEACTIVATED  %0 name\n"
-		"         SESSION DISCONNECTED %0 name %1 host %2 ip %3 port\n"
-		"         TIME                 %4 hour : %5 minute : %6 second\n"
-		"         WEEK                 %2 week\n"
-		"         YEAR                 %0 year\n"
+		"         RECEIVED INPUT        %0 raw text\n"
+		"         RECEIVED LINE         %0 raw text %1 plain text\n"
+		"         RECEIVED OUTPUT       %0 raw text\n"
+		"         RECEIVED PROMPT       %0 raw text %1 plain text\n"
+		"         SCREEN RESIZE         %0 cols %1 rows\n"
+		"         SECOND                %6 second\n"
+		"         SEND OUTPUT           %0 raw text\n"
+		"         SESSION ACTIVATED     %0 name\n"
+		"         SESSION CONNECTED     %0 name %1 host %2 ip %3 port\n"
+		"         SESSION DEACTIVATED   %0 name\n"
+		"         SESSION DISCONNECTED  %0 name %1 host %2 ip %3 port\n"
+		"         SESSION TIMED OUT     %0 name %1 host %2 ip %3 port\n"
+		"         TIME                  %4 hour : %5 minute : %6 second\n"
+		"         VARIABLE UPDATE <VAR> %0 name %1 value\n"
+		"         WEEK                  %2 week\n"
+		"         YEAR                  %0 year\n"
 		"\n"
 		"<178>Example<078>: #event {SESSION CONNECTED} {#read mychar.tin}\n"
 		"\n"
@@ -496,6 +499,7 @@ struct help_type help_table[] =
 		"         #format {test} {%a}   {number}  print corresponding ascii character\n"
 		"         #format {test} {%c}   {string}  use a highlight color name\n"
 		"         #format {test} {%d}   {number}  print a number with integer formatting\n"
+		"         #format {test} {%f}   {string}  perform floating point math\n"
 		"         #format {test} {%g}   {number}  perform thousand grouping on {number}\n"
 		"         #format {test} {%h}   {string}  turn text into a header line\n"
 		"         #format {test} {%l}   {string}  lowercase text\n"
@@ -610,14 +614,15 @@ struct help_type help_table[] =
 		"         from the mud.  Available ANSI color names are:\n"
 		"\n"
 		"         reset, light, faint, underscore, blink, reverse, dim,\n"
+		"\n"
 		"         black, red, green, yellow, blue, magenta, cyan, white,\n"
 		"         b black, b red, b green, b yellow, b blue, b magenta, b cyan, b white\n"
 		"\n"
-		"         Available xterm 256 color names are:\n"
+		"         Available XTERM 256 color names are:\n"
 		"\n"
-		"         dark azure, dark ebony, dark jade, dark lime, dark orange, dark silver,\n"
-		"         dark tan, dark violet, light azure, light ebony, light jade, light lime,\n"
-		"         light orange, light silver, light tan, light violet.\n"
+		"         azure, ebony, jade, lime, orange, pink, silver, tan, violet,\n"
+		"         light azure, light ebony, light jade, light lime, light orange,\n"
+		"         light pink, light silver, light tan, light violet.\n"
 		"\n"
 		"         The %1-99 variables can be used as 'wildcards' that will match with any\n"
 		"         text. They are useful for highlighting a complete line. The %0 variable\n"
@@ -751,6 +756,7 @@ struct help_type help_table[] =
 		"         #list {var} {set} {index} {string}     Change the item at {index}\n"
 		"         #list {var} {size} {variable}          Copy list size to {variable}\n"
 		"         #list {var} {sort} {string}            Insert item in alphabetic order\n"
+		"         #list {var} {tokenize} {string}        Create a character list\n"
 		"\n"
 		"         The index should be between 1 and the list's length. You can also give\n"
 		"         a negative value, in which case -1 equals the last item in the list, -2\n"
@@ -1265,11 +1271,12 @@ struct help_type help_table[] =
 	},
 	{
 		"SESSION",
-		"<178>Command<078>: #session <178>{<078>name<178>}<078> <178>{<078>host<178>}<078> <178>{<078>port<178>}<078>\n"
+		"<178>Command<078>: #session <178>{<078>name<178>} {<078>host<178>} {<078>port<178>} {<078>file<178>}\n"
 		"\n"
-		"         Starts a telnet session with the given name, host and port.  The name\n"
-		"         can be anything you want, except the name of an already existant\n"
-		"         session or a tintin command.\n"
+		"         Starts a telnet session with the given name, host, port, and optional\n"
+		"         file name. The name can be anything you want, except the name of an\n"
+		"         already existant session. If a file name is given the file is only\n"
+		"         read if the session succesfully connects.\n"
 		"\n"
 		"         Without an argument #session shows the currently defined sessions.\n"
 		"\n"
@@ -1483,15 +1490,18 @@ struct help_type help_table[] =
 		"<178>Example<078>: #variable {cool website} {http://tintin.sourceforge.net}\n"
 		"         #chat I was on ${cool website} yesterday!.\n"
 		"\n"
-		"         Variables can be nested using brackets:\n"
+		"         Variables can be nested using brackets or dots:\n"
 		"\n"
 		"<178>Example<078>: #var hp[self] 34;#var hp[target] 46\n"
+		"<178>Example<078>: #var hp.self  34;#var hp.target  46\n"
 		"\n"
 		"         You can see the first nest of a variable using $variable[+1] and the\n"
 		"         last nest using $variable[-1]. Using $variable[-2] will report the\n"
 		"         second last variable, and so on. To show all indices use $variable[].\n"
 		"         To show all values use $variable[%*] or a less generic regex.\n"
 		"\n"
+		"<178>Example<078>: #showme {Targets starting with the letter A: $targets[A%*]\n"
+		"\n"
 		"<178>Comment<078>: To see the internal index of a variable use &<variable name>\n"
 		"\n"
 		"<178>Comment<078>: A non existent nested variable will report itself as 0.\n"
