@@ -35,8 +35,8 @@ DO_COMMAND(do_tick)
 	char arg1[BUFFER_SIZE], arg2[BUFFER_SIZE], arg3[BUFFER_SIZE];
 
 	arg = sub_arg_in_braces(ses, arg, arg1, 0, SUB_VAR|SUB_FUN);
-	arg = get_arg_in_braces(arg, arg2, 1);
-	arg = get_arg_in_braces(arg, arg3, 1);
+	arg = get_arg_in_braces(ses, arg, arg2, 1);
+	arg = get_arg_in_braces(ses, arg, arg3, 1);
 
 	if (*arg3 == 0)
 	{
@@ -78,10 +78,10 @@ DO_COMMAND(do_untick)
 
 DO_COMMAND(do_delay)
 {
-	char arg1[BUFFER_SIZE], arg2[BUFFER_SIZE], arg3[BUFFER_SIZE];
+	char arg1[BUFFER_SIZE], arg2[BUFFER_SIZE], arg3[BUFFER_SIZE], temp[BUFFER_SIZE];
 
 	arg = sub_arg_in_braces(ses, arg, arg1, GET_ONE, SUB_VAR|SUB_FUN);
-	arg = get_arg_in_braces(arg, arg2, 1);
+	arg = get_arg_in_braces(ses, arg, arg2, 1);
 	arg = sub_arg_in_braces(ses, arg, arg3, GET_ALL, SUB_VAR|SUB_FUN);
 
 	if (*arg1 == 0)
@@ -101,15 +101,19 @@ DO_COMMAND(do_delay)
 		{
 			sprintf(arg3, "%lld", utime());
 
-			update_node_list(ses->list[LIST_DELAY], arg3, arg2, arg1);
+			get_number_string(ses, arg1, temp);
 
-			show_message(ses, LIST_TICKER, "#OK, IN {%s} SECONDS {%s} IS EXECUTED.", arg1, arg2);
+			update_node_list(ses->list[LIST_DELAY], arg3, arg2, temp);
+
+			show_message(ses, LIST_TICKER, "#OK, IN {%s} SECONDS {%s} IS EXECUTED.", temp, arg2);
 		}
 		else
 		{
-			update_node_list(ses->list[LIST_DELAY], arg1, arg2, arg3);
+			get_number_string(ses, arg3, temp);
 
-			show_message(ses, LIST_TICKER, "#OK, IN {%s} SECONDS {%s} IS EXECUTED.", arg3, arg2);
+			update_node_list(ses->list[LIST_DELAY], arg1, arg2, temp);
+
+			show_message(ses, LIST_TICKER, "#OK, IN {%s} SECONDS {%s} IS EXECUTED.", temp, arg2);
 		}
 	}
 	return ses;
